

; Handler fr den FIQ:
; Fr den FIQ hat es eine Bank mit separaten Registern R8 ... R14 (R13=SP, R14=LR)
; wenn man nur den separaten Registersatz benutzt,
; braucht man keine Register auf dem Stack zu retten

; dieser Handler bedient die Sound-Ausgabe

; PWM-Peripherie
PWM         EQU   0xE0014000
ofs_PWMIR   EQU   0
ofs_PWMTCR  EQU   0x04
ofs_PWMTC   EQU   0x08
ofs_PWMPR   EQU   0x0C
ofs_PWMPC   EQU   0x10
ofs_PWMMCR  EQU   0x14
ofs_PWMMR0  EQU   0x18
ofs_PWMMR1  EQU   0x1C
ofs_PWMMR2  EQU   0x20
ofs_PWMMR3  EQU   0x24
ofs_PWMMR4  EQU   0x40
ofs_PWMMR5  EQU   0x44
ofs_PWMMR6  EQU   0x48
ofs_PWMPCR  EQU   0x4C
ofs_PWMLER  EQU   0x50


; Flagbits
isSOL  EQU    1

;   0, 1, 2, 3, 6, 10, 15, 21,
; -21, -15, -10, -6, -3, -2, -1, 0
SOLTable3bit
     DCB      0x00
     DCB      0x01
     DCB      0x02
     DCB      0x03
     DCB      0x06
     DCB      0x0a
     DCB      0x0f
     DCB      0x15
     DCB      0xeb
     DCB      0xf1
     DCB      0xf6
     DCB      0xfa
     DCB      0xfd
     DCB      0xfe
     DCB      0xff
     DCB      0x00

; Soundtafel:
;   DCD SamplePointer    Offset 0
;   DCD SampleCount      Offset 4
;   DCD OldSample        Offset 8
;   DCB SoundFlags       Offset 12
;   DCB Slack            Offset 13

    IMPORT      SoundTafel

; Registerverwendung:
; R8  zeigt auf SoundTafel
; R9  SamplePointer
; R10 SampleCount
; R11 Arbeitsregister
; R12 Arbeitsregister

FIQ_Handler
        LDR     R8,=SoundTafel
        LDM     R8,{R9,R10}     ; ldt Samplepointer und SampleCount

	LDRB    R12,[R8,#12]    ; SoundFlags --> R12
        TST     R12,#isSOL
	LDRB    R12,[R9]        ; nchstes Byte --> R12
	BEQ     fiq_wav

; Sound ist im SOL-Format, also 2 Samples/Byte
        LDR     R11,=SOLTable3bit
	TST     R10,#1          ; ob gerade oder nicht
        SUBEQ   R9,R9,#1        ; gerade: Pt zurckstellen
        LSREQ   R12,R12,#4      ; HiNibble --> LoNibble
	AND     R12,R12,#15     ; berflssiges wegschneiden
	LDRSB   R12,[R11,R12]   ; Wert aus SOLTable3bit --> R12 (signed 8-->32 Bit)
        LDR     R11,[R8,#8]     ; OldSample --> R11
	ADD     R12,R12,R11     ; Sample = Sample + OldSample
	STR     R12,[R8,#8]     ; in OldSample merken

; so, in R12 ist das neue Sample
fiq_wav
        LDRB    R11,[R8,#13]          ; Slack laden
        ADD     R12,R12,R11           ; Slack-Offset dazu
	LDR     R11,=PWM
	STR     R12,[R11,#ofs_PWMMR2] ; und Sample ins PWM-Register!
        MOV     R12,#4
	STR     R12,[R11,#ofs_PWMLER] ; bernahme in der nchsten Runde
	MOV     R12,#15
	STR     R12,[R11,#ofs_PWMIR]  ; Interrupt lschen

        ADD     R9,R9,#1              ; SamplePointer weiterstellen
        STR     R9,[R8]               ; und speichern
	SUBS    R10,R10,#1            ; SampleCount herunterzhlen
	STR     R10,[R8,#4]           ; und speichern
        BNE     fiq_done

; Ende des Sounds ist erreicht, also Sound ausstellen
        MOV     R12,#2
	STR     R12,[R11,#ofs_PWMMCR] ; kein Interrupt mehr

fiq_done
        LDR     R12, =VICVectAddr     ; Int lschen
	STR     R11, [R12]
	SUBS    PC,LR,#4              ; return from FIQ
	LTORG


; nur zum Export. Wird von mir nicht aktiv benutzt.
__raise
       MOV      R1, #istRaise
       B        Restart

abort_me
       ORR      R1,R0,#0x80000000      ; Param nach R1
       B        Restart

       LTORG

